function [ POWER_SIG, POWER_NOI, SNR, varargout ] = getSnr( DATA, Fo, FREQ_RESO, TYPE, VALUE, PLOTTING, varargin  )
%GETSNR To get the power of signal, the power of noise and the SNR for each
%       microphone. Beamformed(optimized) SNR and corresponding power of signal
%       and noise can also be provided as variable argument outputs.
%       Plot can be drew if property 'plot' is set to 'on'.
%
%INPUT:
%DATA --- Object data in frequency domain.
%Fo --- Object frequency or frequency band
%FREQ_RESO --- Frequency step. Frequency difference between two bins
%TYPE --- Determine the type of the 5th input argument, 'degree' or 'lag_time'.
%DOA --- Between 1 and 360 (both included) is used to optimize SNR if output arguments require.
%PLOTTING   'on' to turn on plotting; 'off'(default) to turn off plotting.
%('lobe_width') Set a value to bins (default 10) on each single side of main lobe.
%
%OUTPUT:
%POWER_SIG --- The power of signal in frequency Fo for each row of data.
%POWER_NOI --- The power of noise for each row of data.
%SNR --- SNR for each row of data.
%(OPTI_SNR) --- SNR for each row of data after applying DOA.
%(OPTI_SIG) --- The corresponding power of signal to optimized SNR.
%(OPTI_NOI) --- The corresponding power of noise to optimized SNR.



%% Default
load('configuration.mat');
[ mic_num, N ] = size(DATA);
SNR = zeros(1,mic_num);
POWER_SIG = zeros(1,mic_num);
POWER_NOI = zeros(1,mic_num);
OPTI_SNR_SWITCH = 1;
lobe_single_side_bins = 100;



%% Input Arguments Check & Preparation
nVarargin = nargin - 6;
if( rem(nVarargin,2)~=0 )
    warning('Variable input arguments should be in pairs.');
    return;
end
nVarargout = nargout - 3;
if( nVarargout>3 )
    warning('Variable output arguments should not be more than 3.');
    return;
elseif( nVarargout<1 && strcmpi( PLOTTING, 'off') )
    OPTI_SNR_SWITCH = 0;
else
    if( strcmpi(TYPE,'degree') && (VALUE<0 || VALUE>360) )
        warning('The 5th input argument should be within [0,360]');
        return;
    elseif( strcmpi(TYPE,'lag_time') && (abs(max(VALUE))>=N||abs(min(VALUE))>=N) )
        warning('Every element in the 5th input argument should be within (-N,N)');
        return;
    end
end
if( nVarargin>0 )
    if( strcmpi(varargin{1},'lobe_width') )
        lobe_single_side_bins = varargin{2};
    else
        warning(['No such property named ''', varargin{1}], '''.');
        return;
    end
end
[ fo_rows, fo_cols] = size(Fo);
if( fo_rows*fo_cols>2 )
    warning('Fo should be either a single number or a two-element array where the first one is bigger than the second one.');
    return;
end


%% Compute SNR
% For each microphone
for i = 1 :mic_num 
    if( fo_rows*fo_cols==1 )
        POWER_SIG(i) = 0;
        peak_index = 1;
        while( Fo*peak_index<N/2 )
            POWER_SIG(i) = POWER_SIG(i) + 2*sum( abs(DATA(i, round((Fo*peak_index+1)/FREQ_RESO)-lobe_single_side_bins : round((Fo*peak_index+1)/FREQ_RESO)+lobe_single_side_bins)).^2 );
            peak_index = peak_index + 1;
        end
    else
        POWER_SIG(i) = 2*sum( abs(DATA(i, round((Fo(1))/FREQ_RESO)-lobe_single_side_bins: round((Fo(2))/FREQ_RESO)+lobe_single_side_bins)).^2 );
    end
    tot_p = 2*sum( abs(DATA(i, lobe_single_side_bins*3 : round(N/2))).^2 );
    tot_p = getAwayNoisePower( DATA(i,:), tot_p, [9970, 10010], 15, 'off');
    POWER_NOI(i) = tot_p - POWER_SIG(i) ;
    SNR(i) = pow2db( POWER_SIG(i)/POWER_NOI(i) );
    POWER_SIG(i) = pow2db( POWER_SIG(i) );
    POWER_NOI(i) = pow2db( POWER_NOI(i) );
end
% For optimized SNR
if ( OPTI_SNR_SWITCH==1 )
    if( strcmpi(TYPE,'degree') && VALUE~=0 )
        [phase,~,~] = BFphase(conf.Fs, N, VALUE);
        delay = (phase-phase(1))*conf.Fs;
        coef  = (2*pi*delay*(1:N))/N;
        ADJ_DATA  = DATA.*exp(1i * coef);
    elseif( strcmpi(TYPE,'lag_time') )
        t = (1/conf.Fs)*[0:15]/16;
        lagTable = [t,t,t,t];
        adj_data = getAdjusted( DATA, lagTable, 'freq', FREQ_RESO);    %sampling delay
        ADJ_DATA = getAdjusted(adj_data, VALUE, 'freq', FREQ_RESO);    %position delay
    end
%     sumed_adj_data = sum(ADJ_DATA)-ADJ_DATA(18,:)-ADJ_DATA(33,:);
    sumed_adj_data = sum(ADJ_DATA);
    if( fo_rows*fo_cols==1 )
        OPTI_SIG = 0;
        peak_index = 1;
        while( Fo*peak_index<N/2 )
            OPTI_SIG = OPTI_SIG + 2*sum( abs(sumed_adj_data(1, round((Fo*peak_index+1)/FREQ_RESO)-lobe_single_side_bins : round((Fo*peak_index+1)/FREQ_RESO)+lobe_single_side_bins)).^2 );
            peak_index = peak_index + 1;
        end
    else
        OPTI_SIG = 2*sum( abs(sumed_adj_data(1, round((Fo(1))/FREQ_RESO)-lobe_single_side_bins: round((Fo(2))/FREQ_RESO)+lobe_single_side_bins)).^2 );
    end
    tot_p = 2*sum( abs(sumed_adj_data(1, lobe_single_side_bins*3 : round(N/2))).^2 ) ;
    tot_p = getAwayNoisePower( DATA(i,:), tot_p, [9970, 10010], 15, 'off');
    OPTI_NOI = tot_p - OPTI_SIG ;
    OPTI_SNR = pow2db( OPTI_SIG/OPTI_NOI );
    OPTI_SIG = pow2db( OPTI_SIG );
    OPTI_NOI = pow2db( OPTI_NOI );
    varargout{1} = OPTI_SNR;
    varargout{2} = OPTI_SIG;
    varargout{3} = OPTI_NOI;
end


%% Plotting
if( strcmpi(PLOTTING,'on') )
    figure;
    subplot(2,1,1);
    plot( 1:mic_num, POWER_SIG, 'r', ...
          1:mic_num, POWER_NOI, 'b', ...
          1:mic_num, ones(1,mic_num)*OPTI_SIG, 'm',...
          1:mic_num, ones(1,mic_num)*OPTI_NOI, 'g');
    xlabel('Microphones');
    ylabel('Power (dB)');
    legend('Each Signal','Each Noise','Optimized signal', 'Optimized noise');
    title('Power of Signal and Noise');
    subplot(2,1,2);
    plot( 1:mic_num, SNR, 'r', ...
          1:mic_num, ones(1,mic_num)*OPTI_SNR, 'b');
    xlabel('Microphones');
    ylabel('SNR (dB)');
    legend('Each SNR', 'Optimized SNR');
    title('SNR Distribution');
end







end

